#!/bin/bash
VER=1.1
###########################################################
# Tur-FolderLimit by Turranius    (turranius@hotmail.com) #
# Idea by soehest. Go fbsd boy!                           #
#---------------------------------------------------------#
# There are two possible reasons for this script:         #
# 1: You want a nice clean incoming folder with not too   #
#    many folders in them.                                #
# 2: You run two sections on the same partition and want  #
#    to make sure one of the sections does not take up    #
#    too much space.                                      #
#                                                         #
# This script check how many folders there are in the     #
# sections you define. Once there are more then the limit #
# you set, it will move the oldest ones to another        #
# location that you select.                               #
# Works excellent for moving dated dirs too of course.    #
#                                                         #
# Hint: This one checks the time on the folder to consider#
# which folders are the oldest. I suggest you run my      #
# 0DaysDateFix script before this one, so you are sure it #
# moves them in order (For dated dirs only of course).    #
#                                                         #
#---------------------------------------------------------#
# Installation:                                           #
# Copy this script to where ever you want (/glftpd/bin?)  #
# and make it executable.                                 #
# Set up the options below, and be sure you understand    #
# the SECTIONS part.                                      #
#                                                         #
# Run the script with the variable "test"                 #
# ( tur-folderlimit.sh test ) to see what it will do, and #
# that everything looks ok. It will NOT move anything in  #
# test mode, so dont worry.                               #
# Finally, crontab the script to run as often as you like.#
#                                                         #
#---------------------------------------------------------#
# Changelog:                                              #
# 1.1 : Add: Options to preserve modified date on a folder#
# 1.0 :      Initial release.                             #
#---------------------------------------------------------#
# Options:

## Where to log when moving stuff? Set to "" to disable.
LOG="/glftpd/ftp-data/logs/tur-folderlimit.log"

## Sections to move from and to, and how many folders are 
## allowed in the from part.
## One from:to:limit per line. Separate from, to & limit with a :
## The first of the example below means:
## Move the oldest folders from /glftpd/site/0DAYS
## to /glftpd/site/Archive/0DAYS until there are only
## 5 folders left in /glftpd/site/0DAYS

SECTIONS="
/glftpd/site/0DAYS:/glftpd/site/Archive/0DAYS:5
/glftpd/site/MP3:/glftpd/site/Archive/MP3:5
"

## What to exclude in the the sections?
## These folders will not be considered in the count.
EXCLUDE="Today Yesterday GROUPS"

## Preserve Modified time on folders moved?
## If this is TRUE, I will first copy the folder, then
## deleting the old one, instead of just moving.
## TRUE is a bit slower, but better looking result.
PRESERVE="TRUE"

########################################################################
# No changes needed below here.                                        #
########################################################################

## Check date for the logfile.
DATENOW="$(date +%D" - "%T)"

## Make an | between excludes.
EXCLUDE="$( echo $EXCLUDE | tr -s ' ' '|' )"

## Do it.
for section in $SECTIONS; do

  sectionname="$( echo $section | awk -F":" '{print $1}' )"
  sectionto="$( echo $section | awk -F":" '{print $2}' )"
  LIMIT="$( echo $section | awk -F":" '{print $3}' )"
  limitnow="0"

  if [ -e $sectionname ]; then
    if [ "$1" = "test" ]; then
      echo "Moving into $sectionname"
      echo "There will only be $LIMIT folders left."
    fi
  else
    echo "$sectionname can not be found. Check your SECTIONS setup."
    echo "This one is wrong: $section"
    exit 1
  fi

  if [ -e $sectionto ]; then
    if [ "$1" = "test" ]; then
      echo "Stuff will be moved into $sectionto"
    fi
  else
    echo "$sectionto can not be found. Check your SECTIONS setup."
    echo "This one is wrong: $section"
    exit 1
  fi

  cd $sectionname
  DIR="$( ls --sort=time | egrep -vw $EXCLUDE )"
  count="0"
  for dirs in $DIR; do
    count="$( expr $count \+ 1 )"
    if [ "$count" -gt "$LIMIT" ]; then
      if [ "$1" = "test" ]; then
        ## Test mode one. Just printing it to shell.
        echo "Moving $dirs to $sectionto  (test)"
      else
        if [ "$PRESERVE" = "TRUE" ]; then
        ## Copy folder, delete old one.
          cp -rf $dirs $sectionto
          if [ -e $sectionto/$dirs ]; then
            rm -rf $dirs
          else
            echo "Hmm, copying of $dirs to $sectionto seems to have crapped out"
            echo "Im quitting!"
            exit 1
          fi
          if [ "$LOG" != "" ]; then
            echo "$DATENOW : Copying, then removing $dirs into $sectionto" >> $LOG
          fi
        else
          ## Actually move folders, and log if not disabled.
          mv -f $dirs $sectionto
          if [ "$LOG" != "" ]; then
            echo "$DATENOW : Moving $dirs into $sectionto" >> $LOG
          fi
        fi
      fi
    fi
  done

  if [ "$1" = "test" ]; then
    echo ""
  fi
done

exit 0